/*
 * Copyright 2005 Toshiyuki Maeda
 *
 * This file is part of TOS.
 *
 * TOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * TOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "multiboot.h"
#include "kernel.h"

	.text

	.globl	start, _start
start:
_start:
	jmp	physical_entry	/* NOTE: relative addressing */

	.align 4 /* Multiboot header must be aligned to 4 byte. */

	/* Multiboot header */
multiboot_header:
	.long	MULTIBOOT_HEADER_MAGIC
	.long	MULTIBOOT_HEADER_FLAGS
	.long	-(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)
	.long	PHYS(multiboot_header)
	.long	PHYS(_start)
	.long	PHYS(_edata)
	.long	PHYS(_end)
	.long	PHYS(physical_entry)

physical_entry:
	cmpl	$MULTIBOOT_BOOTLOADER_MAGIC, %eax
	jne	die /* NOTE : relative addressing */

	/* set GDT */
	lgdt PHYS(boot_gdt_descr)

	movl $KERNEL_DS, %eax
	movl %eax, %ds
	movl %eax, %es
	movl %eax, %ss

	/* NOTE: We are in 'physical address mode (0x00000000 - )' */
	movl	$(PHYS(boot_stack) + STACK_SIZE), %esp

	/* reset EFLAGS */
	pushl $0
	popfl

	/* enable paging */

	/* NOTE: do not use %ebx (%ebx == &multiboot_information) */

	/* init boot_page_table */
	xor %eax, %eax
	movl $PHYS(boot_ptes), %ecx
	addl $3, %ecx
	movl $PHYS(boot_page_table), %esi
1:
	cmp $256, %eax
	jae 2f

	movl %ecx, (%esi)
	movl %ecx, 3072(%esi)

	addl $1, %eax
	addl $4096, %ecx
	addl $4, %esi

	jmp 1b
2:
	/* init boot_ptes */
	xor %eax, %eax
	movl $3, %ecx
	movl $PHYS(boot_ptes), %esi
3:
	cmp $(1024 * 256), %eax
	jae 4f

	movl %ecx, (%esi)

	addl $1, %eax
	addl $4096, %ecx
	addl $4, %esi

	jmp 3b
4:

	/* set CR3 */
	movl $PHYS(boot_page_table), %eax
	movl %eax, %cr3

	/* set PG bit */
	movl %cr0, %eax
	orl $0x80000000, %eax
	movl %eax, %cr0

	ljmp $KERNEL_CS, $virtual_entry

	.data

	.align	4
boot_gdt_descr:
	.word	GDT_ENTRIES * 8 - 1
	.long	PHYS(gdt_table)


	.comm	boot_stack, STACK_SIZE
	.comm	boot_page_table, PAGE_SIZE, PAGE_SIZE
	.comm	boot_ptes, PAGE_SIZE * 256, PAGE_SIZE
